# -*- coding: utf-8 -*-

import logging
from queue import Empty, Queue
from threading import Event, Lock

from pastepwn.util import start_thread


class ActionHandler(object):

    def __init__(self, action_queue=None, exception_event=None, stop_event=None):
        self.logger = logging.getLogger(__name__)
        self.action_queue = action_queue or Queue()
        self.__exception_event = exception_event or Event()
        self.__stop_event = stop_event or Event()

        self.running = False
        self.__lock = Lock()

    def start(self, ready=None):
        with self.__lock:
            if not self.running:
                self.running = True

                start_thread(self._start, "ActionHandler", self.__exception_event)

            if ready is not None:
                ready.set()

    def _start(self):
        while self.running:
            try:
                # Get paste from queue
                action, paste, analyzer = self.action_queue.get(True, 1)
                self.logger.debug("Performing action '{0}' on paste '{1}' matched by analyzer '{2}'!".format(action.name, paste.key, analyzer.identifier))
                action.perform(paste, analyzer.identifier)
            except Empty:
                if self.__stop_event.is_set():
                    self.logger.debug("orderly stopping ActionHandler")
                    self.running = False
                    break
                elif self.__exception_event.is_set():
                    self.logger.critical("stopping ActionHandler due to exception in another thread")
                    self.running = False
                    break



# -*- coding: utf-8 -*-
from .basicaction import BasicAction
from pastepwn.util import Request


class WebhookAction(BasicAction):
    """Action to perform a Webhook on a matched paste"""
    name = "WebhookAction"

    def __init__(self, url):
        super().__init__()
        self.url = url

    def perform(self, paste, analyzer_name=None):
        """Trigger the webhook"""
        # TODO - More post options ([custom] body, template, choose between GET/POST etc.)
        r = Request()
        r.post(url=self.url)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SHAHashAnalyzer(RegexAnalyzer):
    """Analyzer to match SHA password hashes via regex"""
    name = "SHAHashAnalyzer"

    def __init__(self, action):
        regex = "[a-f0-9]{40}"
        super().__init__(action, regex)



# -*- coding: utf-8 -*-


class BasicAnalyzer(object):
    """Basic analyzer class"""
    name = "BasicAnalyzer"

    def __init__(self, action, identifier=None):
        # The identifier parameter is being used to explicitly identify an analyzer
        self.action = action
        self.identifier = identifier or self.name

    def match(self, paste):
        raise NotImplementedError



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

packages = find_packages(exclude=['tests*'])


# Taken from https://github.com/python-telegram-bot/python-telegram-bot/blob/9d99660ba95b103b3e1dc80414a5ce2fd805260b/setup.py#L9
def requirements():
    """Build the requirements list for this project"""
    requirements_list = []

    with open('requirements.txt') as reqs:
        for install in reqs:
            requirements_list.append(install.strip())

        return requirements_list


with open("README.md", "r", encoding="utf-8") as file:
    readme = file.read()

version_file = os.path.join('pastepwn', 'version.txt')
with open(version_file, "r", encoding="utf-8") as file:
    version = file.readline()

setup(name='pastepwn',
      version=version,
      install_requires=requirements(),
      keywords='python pastebin scraping osint framework',
      description='Python framework to scrape PasteBin pastes and analyze them',
      long_description=readme,
      long_description_content_type='text/markdown',
      url='https://github.com/d-Rickyy-b/pastepwn',
      author='d-Rickyy-b',
      author_email='pastepwn@rickyy.de',
      license='MIT',
      packages=packages,
      include_package_data=True)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class BcryptHashAnalyzer(RegexAnalyzer):
    """Analyzer to match bcrypt password hashes via regex"""
    name = "BcryptHashAnalyzer"

    def __init__(self, action):
        regex = "\$2[ayb]\$.{56}"
        super().__init__(action, regex)



# -*- coding: utf-8 -*-
import logging

import mysql.connector

from .abstractdb import AbstractDB


class MysqlDB(AbstractDB):

    def __init__(self, ip="127.0.0.1", port=27017, dbname="pastepwn", username=None, password=None):
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing MySQLDB - {0}:{1}".format(ip, port))

        self.db = mysql.connector.connect(
            host=ip,
            user=username,
            passwd=password,
            database=dbname
        )

        self.cursor = self.db.cursor()
        self._create_tables()

        self.logger.debug("Connected to database!")

    def _create_tables(self):
        self.cursor.execute("""CREATE TABLE IF NOT EXISTS `pastes` (
                                            `key` TEXT NOT NULL UNIQUE,
                                            `title` TEXT,
                                            `user` TEXT,
                                            `size` INTEGER,
                                            `date` INTEGER,
                                            `expire` INTEGER,
                                            `scrape_url` TEXT,
                                            `full_url` TEXT,
                                            `body` TEXT,
                                            PRIMARY KEY(`key`));""")
        self.db.commit()

    def _insert_data(self, paste):
        self.cursor.execute("INSERT INTO `pastes` (`key`, `title`, `user`, `size`, `date`, `expire`, `syntax`, `scrape_url`, `full_url`, `body`) "
                            "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s);",
                            (paste.key,
                             paste.title,
                             paste.user,
                             paste.size,
                             paste.date,
                             paste.expire,
                             paste.syntax,
                             paste.scrape_url,
                             paste.full_url,
                             paste.body))
        self.db.commit()

    def _get_data(self, key, value):
        pass

    def count(self, key, value):
        # TODO add filter to counting
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def count_all(self):
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def store(self, paste):
        self.logger.debug("Storing paste {0}".format(paste.key))

        try:
            self._insert_data(paste)
        except Exception as e:
            self.logger.debug("Exception '{0}'".format(e))

    def get(self, key):
        return self._get_data("key", key)
